# WARNING - Generated by {fusen} from dev/flat_full.Rmd: do not edit by hand



#' Get Labels for Expected Differential Expression
#'
#' This function assigns labels to genes based on whether their actual effect estimates
#' indicate differential expression according to a given threshold and alternative hypothesis.
#'
#' @param comparison_df A data frame containing comparison results with actual effect estimates.
#' @param coeff_threshold The threshold value for determining differential expression.
#' @param alt_hypothesis The alternative hypothesis for comparison. Possible values are "greater",
#'                      "less", and "greaterAbs".
#' @return A modified data frame with an additional column indicating if the gene is differentially expressed.
#'
#' @examples
#' # Generate a sample comparison data frame
#' comparison_data <- data.frame(
#'   geneID = c("gene1", "gene2", "gene3"),
#'   actual = c(0.5, -0.3, 0.8)
#' )
#'
#' # Get labels for expected differential expression
#' labeled_data <- getLabelExpected(comparison_data, coeff_threshold = 0.2, alt_hypothesis = "greater")
#'
#' @export
getLabelExpected <- function(comparison_df, coeff_threshold, alt_hypothesis) {
  if (alt_hypothesis == "greater") {
    idx_DE <- comparison_df$actual > coeff_threshold
    comparison_df$isDE <- idx_DE
  } else if (alt_hypothesis == "less") {
    idx_DE <- comparison_df$actual < coeff_threshold
    comparison_df$isDE <- idx_DE
  } else if (alt_hypothesis == "greaterAbs") {
    idx_DE <- abs(comparison_df$actual) > coeff_threshold
    comparison_df$isDE <- idx_DE
  }
  ## isDE for random params == NA
  idx_ran_pars <- comparison_df$effect == "ran_pars"
  comparison_df$isDE[idx_ran_pars] <- NA
  return(comparison_df)
}


#' Computes the ROC curve.
#'
#' This function takes a data frame with binary truth values and predicted scores,
#' computes the ROC curve, and returns a data frame containing specificity, sensitivity, and threshold values.
#' This function is inspired by the yardstick package.
#'
#' @param dt A data frame with columns truth (first column) and score (second column).
#' @return A data frame with specificity, sensitivity, and threshold values.
#' @export
compute_roc_curve <- function(dt){
  ## -- replace 0 by minimum machine 
  dt$p.adj[ dt$p.adj == 0 ] <- 1e-217
  pred_obj <- prediction( -log10(dt$p.adj), dt$isDE)
  perf_obj <- performance(pred_obj,"tpr","fpr")
  data2curve <- data.frame(x.name = perf_obj@x.values[[1]], y.name = perf_obj@y.values[[1]])
  names(data2curve) <- c(unname(perf_obj@x.name), unname(perf_obj@y.name))
  return(data2curve)
}


#' Computes area under the ROC curve (AUC).
#'
#' This function calculates the area under the ROC curve (AUC) using specificity and sensitivity values.
#'
#' @param dt A data table with columns for True positive rate and False positive rate
#' @return A numeric value representing the AUC.   
#' @export
compute_roc_auc <- function(dt) Area_Under_Curve(x  = dt$`False positive rate`, y = dt$`True positive rate`)




#' Gets ROC objects for a given parameter.
#'
#' This function takes a data table of evaluation parameters and returns ROC curves for each term
#' and an aggregate ROC curve along with corresponding AUC values.
#'
#' @param evaldata_params Data table containing evaluation parameters.
#' @param col_param Column name specifying the parameter for grouping.
#' @param col_truth Column name for binary ground truth values.
#' @param col_score Column name for predicted scores.
#' @return A list containing ROC curves and AUCs for each group and an aggregate ROC curve and AUC.
#' @importFrom data.table setDT .SD
#' @export
get_roc_object <- function(evaldata_params, col_param = "description", col_truth = "isDE", col_score = "p.adj"  ) {
  
  ## -- subset fixed eff
  evaldata_params <- subset(evaldata_params, effect == "fixed")
  
  ## -- data.table conversion
  dt_evaldata_params <- data.table::setDT(evaldata_params)

  ## -- by params
  roc_curve_params <- dt_evaldata_params[, compute_roc_curve(.SD), by=c("from", col_param), .SDcols=c(col_truth, col_score)]
  roc_auc_params <- roc_curve_params[, compute_roc_auc(.SD), by=c("from", col_param), .SDcols=c("False positive rate", "True positive rate")]
  names(roc_auc_params)[ names(roc_auc_params) == "V1" ] <- "roc_AUC"
  roc_auc_params$roc_randm_AUC <- 0.5

  ## -- aggregate
  roc_curve_agg <- dt_evaldata_params[, compute_roc_curve(.SD), by= "from", .SDcols=c(col_truth, col_score)]
  roc_auc_agg <- roc_curve_agg[, compute_roc_auc(.SD), by="from", .SDcols=c("False positive rate", "True positive rate")]
  names(roc_auc_agg)[ names(roc_auc_agg) == "V1" ] <- "roc_AUC"
  roc_auc_agg$roc_randm_AUC <- 0.5
  
  return(list(byparams = list(roc_curve = as.data.frame(roc_curve_params),
                              roc_auc = as.data.frame(roc_auc_params)),
              aggregate = list(roc_curve = as.data.frame(roc_curve_agg),
                               roc_auc = as.data.frame(roc_auc_agg)))
  )
  
}


#' Builds a ggplot ROC curve.
#'
#' This function takes data frames for ROC curve and AUC and builds a ggplot ROC curve.
#'
#' @param data_curve Data frame with ROC curve.
#' @param data_auc Data frame with AUC.
#' @param palette_color List of colors used.
#' @param ... Additional arguments to be passed to ggplot2::geom_path.
#' @return A ggplot object representing the ROC curve.
#' @export 
build_gg_roc_curve <- function(data_curve, data_auc, palette_color = c("#500472", "#79cbb8") ,  ...){

 
  data_auc <- get_label_y_position(data_auc)
  
  ggplot2::ggplot(data_curve) +
    ggplot2::geom_path(ggplot2::aes(x = `False positive rate` , y = `True positive rate`, ...), linewidth = 1) +
    ggplot2::geom_text(data_auc,
                       mapping = ggplot2::aes(x = 0.75, y = pos_y,
                                              label = paste("AUC :", round(roc_AUC, 2) , sep = ""), col = from)
    ) +
    ggplot2::theme_bw() +
    ggplot2::xlim( 0, 1 ) +
    ggplot2::ylim( 0, 1 ) + 
    ggplot2::scale_color_manual(values = palette_color)
}



#' Computes y-axis position for text labels.
#'
#' This function calculates the y-axis position for text labels in a ggplot based on the levels of a factor.
#' It is specifically designed for use with ROC curve plotting.
#'
#' @param data_auc Data frame with AUC values and factor levels.
#' @return A modified data frame with an additional column pos_y representing y-axis positions.
#' @export
get_label_y_position <- function(data_auc){
  ## -- y text  
  l_y_pos <- c(0.15, 0.05)
  lvls <- levels(as.factor(data_auc$from))
  vec_pos_y <- data_auc$from == lvls[[1]]
  vec_pos_y[vec_pos_y] <- l_y_pos[1]
  vec_pos_y[!vec_pos_y] <- l_y_pos[2]
  data_auc$pos_y <- vec_pos_y
  return(data_auc)
}


#' Gets ROC curves and AUC for both aggregated and individual parameters.
#'
#' This function takes a ROC object and returns ROC curves and AUCs for both aggregated and individual parameters.
#'
#' @param roc_obj ROC object.
#' @param ... Additional arguments to be passed to \code{ggplot2::geom_path}.
#' @return ROC curves and AUCs for both aggregated and individual parameters.
#' @export
get_roc_curve <- function(roc_obj, ...){
  
  ## -- aggreg
  data_curve <- roc_obj$aggregate$roc_curve
  data_auc <- roc_obj$aggregate$roc_auc
  roc_obj$aggregate$roc_curve <- build_gg_roc_curve(data_curve, data_auc, col = from , ... )
  
  ## -- indiv
  data_curve <- roc_obj$byparams$roc_curve
  data_auc <- roc_obj$byparams$roc_auc
  roc_obj$byparams$roc_curve <- build_gg_roc_curve(data_curve, data_auc, col = from,  ... ) +
    ggplot2::facet_wrap(~description) +
    ggplot2::coord_fixed()
  
  return(roc_obj)
}






